
using System;

namespace ODE.Integrators
{
	
	/// <summary>
	/// 2nd order Runge-Kutta integrator
	/// </summary>
	public class RK2Integrator : IntegratorBase
	{
		
		public RK2Integrator()
		{
		}
		
		public override void Update(DerivativesDelegate derivatives) {
			
			double[] k1, k2;
			
			double[] yt = new double[n];
			
			double h2 = h / 2;
			double x2 = x + h2;
			
			// compute intermediary steps
			// get derivatives at x
			derivatives(x, y, out k1);
			
			// use this to compute temporary
			// state at x+h/2
			for (int i = 0; i < n; i++) {
				yt[i] = y[i] + h2*k1[i];
			}
			
			// get derivatives at x+h/2
			derivatives(x2, yt, out k2);
			
			// get final state by using derivatives
			// at x+h/2
			for (int i = 0; i < n; i++) {
				y[i] = y[i] + h*k2[i];
			}
			
			x += h;
		}
	}
}
